
# Dynamic clustering of multivariate panel data
Python code accompanying [Custodio João et al. (2022)](https://doi.org/10.1016/j.jeconom.2022.03.003).


# Table of contents
1. [What is in this repository?](#wat)
2. [How do you run the code](#structure)
3. [Some comments on the GAS filter](#GAS)
4. [Disclaimer](#disclaimer)

# What is in this repository? <a name="wat"></a>
In this repository, you will find all the needed files to replicate the results in [Custodio João et al. (2022)](https://doi.org/10.1016/j.jeconom.2022.03.003).
Simulation is done in Python and for the plotting routines R is employed. 

## main.py
  This is the **main** file of the repository where magic numbers are defined and is the file you need to run to obtain results.
  It simulates data, estimates parameters and writes it to a .csv file with the number of each column representing a simulation-estimation iteration. Every column contains in order the following, where matrices have been vectorized.
  - log-likelihood value
  - status of optimizer convergence
  - estimated parameters
  - restricted parameters (Notice this remains the same in the current set-up)
  - estimated cluster-specific parameters
  - filtered cluster probabilities
  

## mainFunctions.py
  The program is broken down into basically 3 parts: Initialization, Simulation, and Estimation.
  This cycle is repeated for **iNrSimulations** times. 
  The Python file contains 3 methods, that in their turn make use of functions in **auxiliaryFunctions.py** and **helperRoutines.py**:

  - fMaximumLikelihood
  - fSimulateData 
  - fInitializeImportantMatrices

  ## auxiliaryFunctions.py
  
  ## helperRoutines.py
  Contain helper functions that are non-specific to this paper or its methods.
  
  ## prepareDataForPlotting.R
  Read and process the .csv file, saves intermediate results as an .RData object.
  
  ## plottingRoutines.R
  Reads the .RData object and plots the results, as can be found in the Results map.


# How do you run the code? <a name="structure"></a>
To run the code there are a couple of things you need to do.
1. Make sure you have all of the above-mentioned .py and .R files in the same depository.
2. Set the Magic numbers in main.py and pay particular attention to:
   - iRanSeed
   - iNrSimulations
   - iParrallel, default False; increases performance when you have a lot of CPU cores. It is easier to leave this False when trying things out.
   - iSimulationType


# Some comments on the GAS filter <a name="GAS"></a>

To compute the likelihood of the HMM dynamic mixture model, the logistics of the
computations are as follows:

1. starting from initially predicted mixture probabilities, means, and variances for each
mixture component, and setting t=1
2. compute the likelihood contribution for t and add to the likelihood
3. compute the transition probabilities from t to t+1 using the means for time t 
4. compute the score of the time t density to obtain scores using the means and
variances, and use the scores to update the means and covariances to those for t+1
5. use the transition probabilities from step 3 to update the predicted probabilities
to time t+1
6. increase t to t+1 and go to step 2 until all times are processed

# Disclaimer <a name="disclaimer"></a>
***

Disclaimer from http://gasmodel.com/disclaimer.htm: 

The software provided via the links on this site is provided “as is". 
We do not guarantee the software referred to is free from defects. 
You use the software at your own risk. 
We make no warranties as to performance, merchantability, fitness for a particular purpose, or any other warranties whether expressed or implied. 
No oral or written communication from or information provided by us shall create a warranty. 
Under no circumstances shall the providers of the software or the owners of this site be liable for direct, indirect, special, incidental, 
or consequential damages resulting from the use, misuse, or inability to use this software, even if we have been advised of the possibility of such damages.

***