###########

library(xtable)
library(ggplot2)
library(reshape2)
library(tidyr)
library(dplyr)
library(foreach)
library(mlbench)
library(data.table)  # fread function
options(xtable.floating = TRUE)
options(xtable.timestamp = "")

rm(list=ls())
source("eval_sim_functions_v5.r")

#########################################################################################
######## Magic numbers
iSimulationType <- 3
iN <- 100
iT <- 40
iMeanSpecificSmoothParam <- 0
dCircleMidpointDistance = 2
dCircleRadius = 2
iJ <- 2
iD <- 2
dGamma <- 0.25
nr.repetitions <- 1
dSimSemiMajorAxis <- 2
dSimVarianceComponent1 <- 1
dSimVarianceComponent2 <- 1 
dCovariance <- 0.9
iEllipse1Orientation <- 1
iEllipse2Orientation <- 0
dSmoothDGP <- 0.1
dSmoothEstimation <- 0.25
iP <- 2 
iTVprob<- 1  # 0 means constant transition matrix, 1 means tv. transition matrix

simulation_identifiers <- c("iSimulationType","N", "T", "iD", "radius", "dist", "het_A",
                            "iP", "TV", "dGamma", "SMA", "Orient_1",
                            "Orient_2", "Var1", "Var2", 'covariance', 'smoothDGP', 'smoothEst')

simulation_params <-  c(iSimulationType,iN,iT,iD, dCircleRadius, dCircleMidpointDistance,
                        iMeanSpecificSmoothParam,iP, iTVprob, dGamma, dSimSemiMajorAxis,
                        iEllipse1Orientation, iEllipse2Orientation, dSimVarianceComponent1, 
                        dSimVarianceComponent2, dCovariance, dSmoothDGP, dSmoothEstimation)



setting.here <- data.frame(rbind(simulation_params,simulation_params),stringsAsFactors = FALSE)[1,]
colnames(setting.here) <- simulation_identifiers




# As.df: saves estimates of A's
As.df <- as.data.frame(matrix(nrow=0, ncol=nr.repetitions+1+length(simulation_identifiers)))

# trajectories.df: saves the trajectories of the estimated means for plotting
# 2 cols for x and y, 1 for simulation run, 1 for simulation design, 1 for gamma,
#  1 for true/estimated, 1 for time, 1 for cluster 1/2:
trajectories.names <-  c(simulation_identifiers,
                         'true_or_est', 'cluster_idx',
                         't', 'x', 'y', 'total_MSE_over_time', 'total_correct_class', "exit_status")
trajectories.df <- as.data.frame(matrix(nrow=0, ncol=length(trajectories.names)))
names(trajectories.df) <- trajectories.names


#########################################################################################
iK <- iD+(iD^2+iD)/2  ## number of means and unique covariance elements at t
file.name <- paste("Sim_results_3_100_40_0.25_2_2_0_0_2_2_1_0_2_1_0_1_1_0.9_0.25_0.1", ".csv",sep="")

result.coll.1 <- fread(file.name)
# Print optimizer status
print(paste0("File read. # of non-convergence: ", sum(result.coll.1[2,] !=0),
             ". File name: ", file.name))

if (iMeanSpecificSmoothParam == 1){
  A1s <- data.frame(result.coll.1[3:4,])
  A1s <- cbind(data.frame(a_index = c(1, 2), setting.here), A1s)
} else{
  A1s <- data.frame((result.coll.1[3,]))
  A1s <- cbind(data.frame(a_index = 3, setting.here), A1s)
}

#########################################################################################
############################## OUR CLUSTERING ###########################################
#########################################################################################
# WHY ONLY 4?
f_eval_failures <- result.coll.1[2,] == 4
f_eval_failures <- result.coll.1[2,] != 0
sim_with_NAs <- lapply(lapply(result.coll.1, is.na), sum) > 0
total_to_skip <- f_eval_failures | sim_with_NAs
print(paste0(" Our clustering; # of function eval. failures: ",
             sum(f_eval_failures), "; # total skipping due to NA: ", sum(sim_with_NAs)))


# Now as apply function
evals.lss.all <- lapply(result.coll.1[,-..total_to_skip], eval.round, setting.here)

traj_fun_in_nr.simulations <- function(evals.lss, k){
  trajectories_list <- list(estimated.means.cluster1 = evals.lss$estimated.means.cluster1,
                            estimated.means.cluster2 = evals.lss$estimated.means.cluster2,
                            true.means.cluster1 = evals.lss$true.means.cluster1,
                            true.means.cluster2 = evals.lss$true.means.cluster2)
  trajectories.here <- do.call(rbind, lapply(trajectories_list, t))
  
  trajectories.df.this.run <- data.frame(c(setting.here),
                                         rep(0:1, each=setting.here$T*2),
                                         rep(rep(1:2, each=setting.here$T), 2),
                                         rep(1:setting.here$T, 4),
                                         trajectories.here[,1], trajectories.here[,2],
                                         rep(evals.lss$MSE.p_t, 4),
                                         rep(evals.lss$correct.class_per_time, 4),
                                         rep(evals.lss$exit_status, (setting.here$T)*4))
  names(trajectories.df.this.run) <- names(trajectories.df)
  
  if (setting.here$iP >2){
    n_of_extra_clusters <- setting.here$iP - 2
    extra_trajectories <- do.call(rbind, lapply(evals.lss$other_cluster_estimated_means, t))
    extra_clusters <- data.frame(c(setting.here),
                                 rep(0, setting.here$T*n_of_extra_clusters),
                                 rep(3:setting.here$iP, each=setting.here$T),
                                 rep(1:setting.here$T, n_of_extra_clusters),
                                 extra_trajectories,
                                 rep(evals.lss$MSE.p_t, n_of_extra_clusters),
                                 rep(evals.lss$correct.class_per_time, n_of_extra_clusters),
                                 rep(evals.lss$exit_status, (setting.here$T)*n_of_extra_clusters))
    names(extra_clusters) <- names(trajectories.df)
    trajectories.df.this.run <- rbind(trajectories.df.this.run, extra_clusters)
  }
  return(trajectories.df.this.run)
}

evals_fun_in_nr.simulations <- function(evals.lss){
  return(c(evals.lss$gamma.hat, evals.lss$correct.class,
                        evals.lss$average.MSE, evals.lss$likelihood))
}

tmp_trajectories <- mapply(traj_fun_in_nr.simulations,
                           evals.lss.all, SIMPLIFY=FALSE)

for (sim_i in 1:length(tmp_trajectories)){
  tmp_trajectories[[sim_i]]$sim_run <- gsub("Var", "Sim", names(tmp_trajectories)[sim_i])
}
trajectories_h <- bind_rows(tmp_trajectories)
trajectories_h$sim_run <- factor(trajectories_h$sim_run)

evals.table.1 <- t(bind_rows(lapply(evals.lss.all, evals_fun_in_nr.simulations)))

#########################################################################################
############################## Hierarchical #############################################
#########################################################################################
print(paste0(" Hierarchical clustering; # of function eval. failures: ",
             sum(f_eval_failures), "; # total skipping due to NA: ", sum(sim_with_NAs)))

evals.lss.all.ward <- lapply(result.coll.1[,-..total_to_skip], eval.hierarchical, setting.here)
evals_fun_in_nr.simulations.ward <- function(evals.lss){
  return(c(evals.lss$class.outcome, evals.lss$average.MSE))
}
evals.table.1.ward <- t(bind_rows(lapply(evals.lss.all.ward,
                                         evals_fun_in_nr.simulations.ward)))

#########################################################################################


all.evals.table.h <- data.frame(c(setting.here), 1:nrow(evals.table.1), evals.table.1, evals.table.1.ward)
names(all.evals.table.h)[(length(setting.here)+1):ncol(all.evals.table.h)] <- 
  c('simulation', 'hat_gamma', 'class', 'MSE', 'LL', 'class_ward', 'MSE_ward')


dfs <- list(
  all.evals.table.h,
  trajectories_h,
  A1s)


combine_dfs <- function(df_list){
  res <- list()
  for(k in 1:ncol(df_list)){
    tmp <- list()
    for (i in 1:nrow(df_list)){
      tmp <- c(tmp, df_list[i, k])
    }
    res[[k]] <- bind_rows(tmp)
  }
  return(res)
}


results <- combine_dfs(rbind(dfs))

print("Saving")
save.image(paste0("subset_sim_complete.RData"))
